;
; Title screen and menu support for nametable editor
; Copyright 2012 Damian Yerrick
;
; Copying and distribution of this file, with or without
; modification, are permitted in any medium without royalty provided
; the copyright notice and this notice are preserved in all source
; code copies.  This file is offered as-is, without any warranty.
;
.include "src/nes.h"
.include "src/ram.h"

.export puts_multiline
.import read_pads

LF = $0A

;;
; @param A high byte of string
; @param Y low byte of string
.proc puts_multiline
srcLo = $00
srcHi = $01
dstLo = $02
dstHi = $03

  sta srcHi
  ldx #0
  stx srcLo  ; keep the low byte in Y

newline:
  lda dstHi
  sta PPUADDR
  lda dstLo
  sta PPUADDR
  clc
  adc #32
  sta dstLo
  bcc :+
  inc dstHi
:
  ldx #0
charloop:
  lda (srcLo),y
  beq done
  iny
  bne :+
  inc srcHi
:
  cmp #LF
  beq newline
  sta PPUDATA
  jmp charloop
done:
  rts
.endproc

;;
; Loads a colorset 0 compatible with the current background color
; and leaves VRAM address at $3F04.
.proc load_colorset_0
  lda #$3F
  sta PPUADDR
  lda #$00
  sta PPUADDR

  ; get lightness in upper nibble
  lda $7F00
  and #$3F
  sta PPUDATA
  eor #$0F
  clc
  adc #$01  ; 00->10, 01-0C->0F-04

  cmp #$20  ; if C is set, we need to use black
  lda #$20  ; white
  bcc not_black
  lda #$0F
not_black:
  sta PPUDATA
  sta PPUDATA
  sta PPUDATA
  rts
.endproc

; Title screen ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.proc title_screen
  lda #VBLANK_NMI
  sta PPUCTRL
  jsr load_colorset_0
  lda #' '
  ldx #$20
  ldy #$00
  jsr ppu_clear_nt
  lda #$20
  sta $03
  lda #$E2
  sta $02
  lda #>title_msg
  ldy #<title_msg
  jsr puts_multiline

loop:
  lda nmis
:
  cmp nmis
  beq :-
  ldx #0
  ldy #0
  lda #VBLANK_NMI
  clc
  jsr ppu_screen_on

  ; Start: hide the title screen and go onto the next phase
  jsr read_pads
  lda new_keys
  and #KEY_START
  beq not_start
  ldx #SCREEN_CHRPICKER
  rts
not_start:

  ; Left+B+Select: Reformat SRAM
  lda cur_keys
  cmp #KEY_LEFT|KEY_B|KEY_SELECT
  bne loop
  lda new_keys
  cmp #KEY_SELECT
  bne loop

  ; TO DO: Confirm reformat SRAM
  rts
.endproc

.segment "RODATA"
title_msg:
  .byt "GRAPHICS EDITOR FOR NES",10
  .byt "COPR.2012 DAMIAN YERRICK",10,10,10
  .byt "PRESS START BUTTON TO DRAW!",10,10,10,10,10
  .byt "____________________________",10,10
  .byt "NOTICE:",10
  .byt "THIS GAME PAK SAVES YOUR",10
  .byt "PICTURE WITH A BATTERY",10
  .byt "CIRCUIT. TO AVOID SMUDGES,",10
  .byt "ALWAYS HOLD THE RESET BUTTON",10
  .byt "WHILE TURNING OFF THE POWER.",10,10
  .byt "TO ERASE THE STORED PICTURE,",10
  .byt "HOLD LEFT+B AND PRESS SELECT",0

reformat_confirm_msg:
  .byt "THE PICTURE YOU DREW IS",10
  .byt "ABOUT TO BE ERASED. IF YOU",10
  .byt "AREN'T SURE YOU WANT TO DO",10
  .byt "THIS, PRESS THE RESET",10
  .byt "BUTTON NOW.",0

.segment "CODE"

; Drawing help ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.proc comingsoon1_screen
  lda #>comingsoon1_msg
  ldy #<comingsoon1_msg
  jmp static_screen
.endproc

.proc drawing_help_screen
  lda #>drawing_help_msg
  ldy #<drawing_help_msg
.endproc
.proc static_screen
  STA $FF
  pha
  tya
  pha
  lda #VBLANK_NMI
  sta PPUCTRL
  asl a
  sta PPUMASK
  jsr load_colorset_0
  lda #' '
  ldx #$20
  ldy #$00
  jsr ppu_clear_nt
  lda #$20
  sta $03
  lda #$62
  sta $02
  pla
  tay
  pla
  jsr puts_multiline

loop:
  lda nmis
:
  cmp nmis
  beq :-
  ldx #0
  ldy #0
  lda #VBLANK_NMI
  clc
  jsr ppu_screen_on

  ; Start: hide the title screen and go onto the next phase
  jsr read_pads
  lda new_keys
  beq loop
  ldx #SCREEN_CHRPICKER
  rts
.endproc

.segment "RODATA"
drawing_help_msg:
  .incbin "src/drawing_help.txt"
  .byt 0
comingsoon1_msg:
  .incbin "src/comingsoon1.txt"
  .byt 0
.segment "CODE"

